/* www.N-Modellbahn.de
  Created by S.Schwarz
  Stellpult control with Millis Methode and INPUT_PULL
  Crossing light control with Millis Methode, pushbutton flip flop and INPUT_PULL

  History:
  V3: Einbau Flankenschutz Weiche 3 bei schalten Weiche 2 in gerade Richtung
  
*/

#include <EEPROM.h>
#include <Servo.h>

//##########################  Crossing control  ######################################

// **********************Crossing light PIN********************

//Definition of input PIN for Crossing light control pushbutton
const int  Button_inCrossing = 31;                        // the number of the crossing light pushbutton pin

// Definition of output PINs for Crossing light LEDs:
const int Led_outCrossing1 =  52;                       // the number of the Crossing light 1 LED pin
const int Led_outCrossing2 =  53;                       // the number of the Crossing light 1 LED pin

// **********************Variables for Crossing light LED status********************
int Led_stCrossing1 = LOW;                            // variable for Crossing light 1 LED status
int Led_stCrossing2 = LOW;                            // variable for Crossing light 2 LED status

// **********************Variables for Crossing light pushbutton status********************
int Button_stCrossing = 0;                            // variable for reading the Crossing pushbutton status
int Blink_stCrossing = 1;                             // Variable for Crossing Blinking status
int Button_stCrossing_Previous = 0;                   // variable for storing last Crossing pushbutton satus

// **********************Variables for timers********************
unsigned long Crossing_tiCurrentMillis = 0;           // variable for reading the current millis value
unsigned long Crossing_tiPreviousMillis = 0;          // variable to store the previous millis value
unsigned long Crossing_tiLightInterval = 500;         // Interval timer for Crossing light flashing



//########################  Deflector control  #########################################

// **********************Deflector PIN********************

// Definition of constants. They're used here to set pin numbers:
//Definition of input PINs for Deflector control pushbutton
const int Button_inStartLft = 22;      // the number of the Start track left pushbutton pin
const int Button_inStartRgt = 23;      // the number of the Start track right pushbutton pin
const int Button_inGleis1 = 24;        // the number of the track 1 pushbutton pin
const int Button_inGleis2 = 25;        // the number of the track 2 pushbutton pin
const int Button_inGleis3 = 26;        // the number of the track 3 pushbutton pin

// Definition of output PINs for Deflector indication LEDs:
const int Led_outWeiche1Straight = 32;   // the number of the Deflector 1 Straight LED pin
const int Led_outWeiche1Turn = 33;       // the number of the Deflector 1 Turn LED pin
const int Led_outWeiche2Straight = 34;   // the number of the Deflector 2 Straight LED pin
const int Led_outWeiche2Turn = 35;       // the number of the Deflector 2 Turn LED pin
const int Led_outWeiche3Straight = 36;   // the number of the Deflector 3 Straight LED pin
const int Led_outWeiche3Turn = 37;       // the number of the Deflector 3 Turn LED pin

// Definition of output PINs for Deflector Relays:
const int Relay_outWeiche1Straight = 38;     // the number of the Deflector 1 Straight Relay pin
const int Relay_outWeiche1Turn = 39;         // the number of the Deflector 1 Turn Relay pin
const int Relay_outWeiche2Straight = 40;     // the number of the Deflector 2 Straight Relay pin
const int Relay_outWeiche2Turn = 41;         // the number of the Deflector 2 Turn Relay pin
const int Relay_outWeiche3Straight = 42;     // the number of the Deflector 3 Straight Relay pin
const int Relay_outWeiche3Turn = 43;         // the number of the Deflector 3 Turn Relay pin

// **********************Variables for Pushbutton status********************

// variables for reading Deflector pushbutton status:
int Button_stStartLft = 1;              // variable for reading the Start track left pushbutton
int Button_stStartRgt = 1;              // variable for reading the Start track right pushbutton
int Button_stGleis1 = 1;                // variable for reading the track 1 pushbutton
int Button_stGleis2 = 1;                // variable for reading the track 2 pushbutton
int Button_stGleis3 = 1;                // variable for reading the track 3 pushbutton

// **********************Variables for LED status********************
// variables for reading Deflector LED status:
int Led_stWeiche1Straight = 0;              // variable for reading the Deflector 1 Straight LED pin
int Led_stWeiche1Turn = 0;                  // variable for reading the Deflector 1 Turn LED pin
int Led_stWeiche2Straight = 0;              // variable for reading the Deflector 2 Straight LED pin
int Led_stWeiche2Turn = 0;                  // variable for reading the Deflector 2 Turn LED pin
int Led_stWeiche3Straight = 0;              // variable for reading the Deflector 3 Straight LED pin
int Led_stWeiche3Turn = 0;                  // variable for reading the Deflector 3 Turn LED pin

// **********************Variables for LED status********************
int IfPath_st1 = 0;                           // variable for reading ifPath status
int IfPath_st2 = 0;                           // variable for reading ifPath status
int IfPath_st3 = 0;                           // variable for reading ifPath status
int IfPath_st4 = 0;                           // variable for reading ifPath status
int IfPath_st5 = 0;                           // variable for reading ifPath status
int IfPath_st6 = 0;                           // variable for reading ifPath status
int IfPath_st7 = 0;                           // variable for reading ifPath status
int IfPath_st8 = 0;                           // variable for reading ifPath status

// **********************Variables for timers********************
// variables for reading the millis () value
unsigned long  Time_tilast1 = 0;               // variable for reading the millis value 1
unsigned long  Time_tilast2 = 0;               // variable for reading the millis value 2
unsigned long  Time_tilast3 = 0;               // variable for reading the millis value 3
unsigned long  Time_tilast4 = 0;               // variable for reading the millis value 4
unsigned long  Time_tilast5 = 0;               // variable for reading the millis value 5
unsigned long  Time_tilast6 = 0;               // variable for reading the millis value 6
unsigned long  Time_tilast7 = 0;               // variable for reading the millis value 7
unsigned long  Time_tilast8 = 0;               // variable for reading the millis value 7

const int DelayTimeWeiche = 500;               // Delay timer duration
const int DelayTimeServo = 100;                //Delay time duration to control servo speed

// **********************Variables for EEPROM********************
int EEprom_id0 = 0;
int EEprom_id1 = 1;
int EEprom_id2 = 2;
int EEprom_id3 = 3;
int EEprom_id4 = 4;
int EEprom_id5 = 5;
int EEprom_id6 = 6;
int EEprom_id7 = 7;
int EEprom_id8 = 8;
int EEprom_id9 = 9;
int EEprom_id10 = 10;
int EEprom_id11 = 11;


void setup() {
  //##########################  Crossing control  ######################################

  //*******************Crossing PIN Mode******************

  // initialize the Crossing Light pushbutton pin as an input:
  pinMode(Button_inCrossing, INPUT_PULLUP);

  // initialize the Crossing Light LED pins as an output:
  pinMode(Led_outCrossing1, OUTPUT);
  pinMode(Led_outCrossing2, OUTPUT);

  //########################  Deflector control  #########################################

  // initialize the Deflector pushbutton pin as an input:
  pinMode(Button_inStartLft, INPUT_PULLUP);
  pinMode(Button_inStartRgt, INPUT_PULLUP);
  pinMode(Button_inGleis1, INPUT_PULLUP);
  pinMode(Button_inGleis2, INPUT_PULLUP);
  pinMode(Button_inGleis3, INPUT_PULLUP);

  // initialize the Deflector LED pins as an output:
  pinMode(Led_outWeiche1Straight, OUTPUT);
  pinMode(Led_outWeiche1Turn, OUTPUT);
  pinMode(Led_outWeiche2Straight, OUTPUT);
  pinMode(Led_outWeiche2Turn, OUTPUT);
  pinMode(Led_outWeiche3Straight, OUTPUT);
  pinMode(Led_outWeiche3Turn, OUTPUT);

  // initialize the Deflector Relay pins as an output:
  pinMode(Relay_outWeiche1Straight, OUTPUT);
  pinMode(Relay_outWeiche1Turn, OUTPUT);
  pinMode(Relay_outWeiche2Straight, OUTPUT);
  pinMode(Relay_outWeiche2Turn, OUTPUT);
  pinMode(Relay_outWeiche3Straight, OUTPUT);
  pinMode(Relay_outWeiche3Turn, OUTPUT);

  //initialize the Deflector Relay pins status:
  digitalWrite(Relay_outWeiche1Straight, HIGH);
  digitalWrite(Relay_outWeiche1Turn, HIGH);
  digitalWrite(Relay_outWeiche2Straight, HIGH);
  digitalWrite(Relay_outWeiche2Turn, HIGH);
  digitalWrite(Relay_outWeiche3Straight, HIGH);
  digitalWrite(Relay_outWeiche3Turn, HIGH);


  //*******************Init LED Status from EEPROM******************
  checkLedState();

}

void loop() {
  //##########################  Crossing control  ######################################

  
  // ########### Crossing LED control: ###########
 
  Crossing_tiCurrentMillis = millis();                                                            // Write actual time in crossing_tiCurrentMillis

  if ( Blink_stCrossing == HIGH) {                                                                // check if Blink_stCrossing is active, if yes activate crossing LEDs
    if (Crossing_tiCurrentMillis - Crossing_tiPreviousMillis > Crossing_tiLightInterval) {        // check if delay time to switch other LEDs is elapsed
      Crossing_tiPreviousMillis = Crossing_tiCurrentMillis;                                       // write actual time to Crossing_tiPreviousMillis
      if (Led_stCrossing1 == LOW) {                                                               // check if LED status is LOW
        Led_stCrossing1 = HIGH;                                                                   // Set Led_stCrossing1 to High
        Led_stCrossing2 = LOW;                                                                    // Set Led_stCrossing2 to low
      } else {
        Led_stCrossing1 = LOW;                                                                    // Set Led_stCrossing1 to low
        Led_stCrossing2 = HIGH;                                                                   // Set Led_stCrossing2 to High
      }
      digitalWrite(Led_outCrossing1, Led_stCrossing1);                                            // Switch LED Crossing 1 corresponding to Led_stCrossing1)
      digitalWrite(Led_outCrossing2, Led_stCrossing2);                                            // Switch LED Crossing 2 corresponding to Led_stCrossing2)
    }
  } else {
    digitalWrite(Led_outCrossing1, LOW);                                                          // Switch LED Crossing 1 off                                                      
    digitalWrite(Led_outCrossing2, LOW);                                                          // Switch LED Crossing 2 off  
  }

  // ########### Check Crossing pushbutton status for Flip Flop: ###########
  Button_stCrossing = digitalRead (Button_inCrossing);                                           // Check crossing pushbutton Status
  if (Button_stCrossing_Previous != Button_stCrossing) {                                         // Check if crossing pushbutton Status has changed
    if (Button_stCrossing == HIGH)                                                               // If pushbutton was activated, Blink_stCrossing will be changed
    {

      Blink_stCrossing = !Blink_stCrossing;                 

    }
    Button_stCrossing_Previous = Button_stCrossing;                                               // Actual Crossing pushbutton Status is stored
    delay (200);                                                                                  // Debouncing for the pushbutton to avoid toggle of signal   
  }

  //########################  Deflector control  #########################################

  // ########### read the state of the Deflector pushbutton value: ###########
  Button_stStartLft = digitalRead(Button_inStartLft);
  Button_stStartRgt = digitalRead(Button_inStartRgt);
  Button_stGleis1 = digitalRead(Button_inGleis1);
  Button_stGleis2 = digitalRead(Button_inGleis2);
  Button_stGleis3 = digitalRead(Button_inGleis3);


  // ########### Deflector control: ###########

  if ((Button_stStartLft == LOW) && (Button_stGleis1 == LOW))      // check if the Start left & Gleis 1 Button is pressed
  {
    digitalWrite(Relay_outWeiche1Turn, LOW);                      // Switch Relay ON for Weiche 1 Turn position
    digitalWrite(Led_outWeiche1Turn, HIGH);                       // Switch LED ON for Weiche 1 Turn position indication
    digitalWrite(Led_outWeiche1Straight, LOW);                    // Switch LED OFF for Weiche 1 Straight position indication
    Led_stWeiche1Straight = 0;                                    // Status LED Weiche 1 Straight position indication for EEPROM
    Led_stWeiche1Turn = 1;                                        // Status LED Weiche 1 Turn position indication for EEPROM
    EEPROM.update(EEprom_id0, Led_stWeiche1Straight);             // Status LED Weiche 1 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id1, Led_stWeiche1Turn);                 // Status LED Weiche 1 turned position stored in EEPROM if value different to already stored value
    Time_tilast1 = millis();                                      // Write actual time in Time_tilast1
    IfPath_st1 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast1) >= DelayTimeWeiche) && (IfPath_st1 == 1))       //check if delay time to switch off the relay is elapsed & IfPath is true
  {
    digitalWrite(Relay_outWeiche1Turn, HIGH);                     // Switch Relay OFF for Weiche 1 Turn position
    IfPath_st1 = 0;                                                // Set IfPath back to 0
  }

  if ((Button_stStartLft == LOW) && (Button_stGleis2 == LOW))     // check if the Start left & Gleis 2 Button is pressed
  {
    digitalWrite(Relay_outWeiche1Straight, LOW);                  // Switch Relay ON for Weiche 1 Straight position
    digitalWrite(Led_outWeiche1Straight, HIGH);                   // Switch LED ON for Weiche 1 Straight position indication
    digitalWrite(Led_outWeiche1Turn, LOW);                        // Switch LED OFF for Weiche 1 Turn position indication
    Led_stWeiche1Straight = 1;                                    // Status LED Weiche 1 Straight position indication for EEPROM
    Led_stWeiche1Turn = 0;                                        // Status LED Weiche 1 Turn position indication for EEPROM
    EEPROM.update(EEprom_id0, Led_stWeiche1Straight);             // Status LED Weiche 1 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id1, Led_stWeiche1Turn);                 // Status LED Weiche 1 turned position stored in EEPROM if value different to already stored value
    Time_tilast2 = millis();                                      // Write actual time in Time_tilast2
    IfPath_st2 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast2) >= DelayTimeWeiche) && (IfPath_st2 == 1))            //check if delay time to switch off the relay is elapsed & IfPath is true
  {
    digitalWrite(Relay_outWeiche1Straight, HIGH);                 // Switch Relay OFF for Weiche 1 Straight position
    IfPath_st2 = 0;                                               // Set IfPath back to 0
  }

  if ((Button_stStartLft == LOW) && (Button_stGleis3 == LOW))     // check if the Start left & Gleis 3 Button is pressed
  {
    digitalWrite(Relay_outWeiche1Turn, LOW);                      // Switch Relay ON for Weiche 1 Turn position
    digitalWrite(Led_outWeiche1Turn, HIGH);                       // Switch LED ON for Weiche 1 Turn position indication
    digitalWrite(Led_outWeiche1Straight, LOW);                    // Switch LED OFF for Weiche 1 Straight position indication
    Led_stWeiche1Straight = 0;                                    // Status LED Weiche 1 Straight position indication for EEPROM
    Led_stWeiche1Turn = 1;                                        // Status LED Weiche 1 Turn position indication for EEPROM
    EEPROM.update(EEprom_id0, Led_stWeiche1Straight);             // Status LED Weiche 1 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id1, Led_stWeiche1Turn);                 // Status LED Weiche 1 turned position stored in EEPROM if value different to already stored value
    Time_tilast3 = millis();                                      // Write actual time in Time_tilast3
    IfPath_st3 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast3) >= DelayTimeWeiche) && (IfPath_st3 == 1))             //check if delay time to switch off the relay is elapsed
  {
    digitalWrite(Relay_outWeiche1Turn, HIGH);                     // Switch Relay OFF for Weiche 1 Straight position
    digitalWrite(Relay_outWeiche3Straight, LOW);                  // Switch Relay ON for Weiche 3 Straight position
    digitalWrite(Led_outWeiche3Straight, HIGH);                   // Switch LED ON for Weiche 3 Straight position indication
    digitalWrite(Led_outWeiche3Turn, LOW);                        // Switch LED OFF for Weiche 3 Turn position indication
    Led_stWeiche3Straight = 1;                                    // Status LED Weiche 3 Straight position indication for EEPROM
    Led_stWeiche3Turn = 0;                                        // Status LED Weiche 3 Turn position indication for EEPROM
    EEPROM.update(EEprom_id4, Led_stWeiche3Straight);             // Status LED Weiche 3 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id5, Led_stWeiche3Turn);                 // Status LED Weiche 3 turned position stored in EEPROM if value different to already stored value
    IfPath_st3 = 0;                                               // Set IfPath back to 0
    Time_tilast4 = millis();                                      // Write actual time in Time_tilast4
    IfPath_st4 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast4) >= DelayTimeWeiche) && (IfPath_st4 == 1))           //check if delay time to switch off the relay is elapsed
  {
    digitalWrite(Relay_outWeiche3Straight, HIGH);                 // Switch Relay OFF for Weiche 3 Straight position
    IfPath_st4 = 0;                                               // Set IfPath back to 0
  }

  if ((Button_stStartRgt == LOW) && (Button_stGleis1 == LOW))     // check if the Start right & Gleis 1 Button is pressed
  {
    digitalWrite(Relay_outWeiche2Turn, LOW);                      // Switch Relay ON for Weiche 2 Turn position
    digitalWrite(Led_outWeiche2Turn, HIGH);                       // Switch LED ON for Weiche 2 Turn position indication
    digitalWrite(Led_outWeiche2Straight, LOW);                    // Switch LED OFF for Weiche 2 Straight position indication
    Led_stWeiche2Straight = 0;                                    // Status LED Weiche 2 Straight position indication for EEPROM
    Led_stWeiche2Turn = 1;                                        // Status LED Weiche 2 Turn position indication for EEPROM
    EEPROM.update(EEprom_id2, Led_stWeiche2Straight);             // Status LED Weiche 2 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id3, Led_stWeiche2Turn);                 // Status LED Weiche 2 turned position stored in EEPROM if value different to already stored value
    Time_tilast5 = millis();                                      // Write actual time in Time_tilast5
    IfPath_st5 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast5) >= DelayTimeWeiche) && (IfPath_st5 == 1))             //check if delay time to switch off the relay is elapsed
  {
    digitalWrite(Relay_outWeiche2Turn, HIGH);                     // Switch Relay OFF for Weiche 2 Turn position
    digitalWrite(Relay_outWeiche3Turn, LOW);                      // Switch Relay ON for Weiche 3 Turn position
    digitalWrite(Led_outWeiche3Turn, HIGH);                       // Switch LED ON for Weiche 3 Turn position indication
    digitalWrite(Led_outWeiche3Straight, LOW);                    // Switch LED OFF for Weiche 3 Straight position indication
    Led_stWeiche3Straight = 0;                                    // Status LED Weiche 3 Straight position indication for EEPROM
    Led_stWeiche3Turn = 1;                                        // Status LED Weiche 3 Turn position indication for EEPROM
    EEPROM.update(EEprom_id4, Led_stWeiche3Straight);             // Status LED Weiche 3 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id5, Led_stWeiche3Turn);                 // Status LED Weiche 3 turned position stored in EEPROM if value different to already stored value
    IfPath_st5 = 0;                                               // Set IfPath back to 0
    Time_tilast6 = millis();                                      // Write actual time in Time_tilast6
    IfPath_st6 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  if (((millis() - Time_tilast6) >= DelayTimeWeiche) && (IfPath_st6 == 1))           //check if delay time to switch off the relay is elapsed
  {
    digitalWrite(Relay_outWeiche3Turn, HIGH);                     // Switch Relay OFF for Weiche 2 Turn position
    IfPath_st6 = 0;                                               // Set IfPath back to 0
  }

 if ((Button_stStartRgt == LOW) && (Button_stGleis2 == LOW))     // check if the Start right & Gleis 2 Button is pressed
  {
    digitalWrite(Relay_outWeiche2Straight, LOW);                  // Switch Relay ON for Weiche 2 Straight position
    digitalWrite(Led_outWeiche2Straight, HIGH);                   // Switch LED ON for Weiche 2 Straight position indication
    digitalWrite(Led_outWeiche2Turn, LOW);                        // Switch LED OFF for Weiche 2 Turn position indication
    Led_stWeiche2Straight = 1;                                    // Status LED Weiche 2 Straight position indication for EEPROM
    Led_stWeiche2Turn = 0;                                        // Status LED Weiche 2 Turn position indication for EEPROM
    EEPROM.update(EEprom_id2, Led_stWeiche2Straight);             // Status LED Weiche 2 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id3, Led_stWeiche2Turn);                 // Status LED Weiche 2 turned position stored in EEPROM if value different to already stored value
    Time_tilast7 = millis();                                      // Write actual time in Time_tilast7
    IfPath_st7 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }

  if (((millis() - Time_tilast7) >= DelayTimeWeiche) && (IfPath_st7 == 1)) //check if delay time to switch off the relay is elapsed
    {
    digitalWrite(Relay_outWeiche2Straight, HIGH);                     // Switch Relay OFF for Weiche 2 Turn position
    digitalWrite(Relay_outWeiche3Straight, LOW);                      // Switch Relay ON for Weiche 3 Turn position
    digitalWrite(Led_outWeiche3Straight, HIGH);                       // Switch LED ON for Weiche 3 Turn position indication
    digitalWrite(Led_outWeiche3Turn, LOW);                    // Switch LED OFF for Weiche 3 Straight position indication
    Led_stWeiche3Straight = 1;                                    // Status LED Weiche 3 Straight position indication for EEPROM
    Led_stWeiche3Turn = 0;                                        // Status LED Weiche 3 Turn position indication for EEPROM
    EEPROM.update(EEprom_id4, Led_stWeiche3Straight);             // Status LED Weiche 3 Straight position stored in EEPROM if value different to already stored value
    EEPROM.update(EEprom_id5, Led_stWeiche3Turn);                 // Status LED Weiche 3 turned position stored in EEPROM if value different to already stored value
    IfPath_st7 = 0;                                               // Set IfPath back to 0
    Time_tilast8 = millis();                                      // Write actual time in Time_tilast6
    IfPath_st8 = 1;                                               // Write status of path = 1 (needed to ensure proper working of millis function)
  }
  
   if (((millis() - Time_tilast8) >= DelayTimeWeiche) && (IfPath_st8 == 1))           //check if delay time to switch off the relay is elapsed   
  {
    digitalWrite(Relay_outWeiche3Straight, HIGH);                 // Switch Relay OFF for Weiche 2 Straight position
    IfPath_st8 = 0;                                               // Set IfPath back to 0
   }

}
